"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useGraphiQL = exports.renderGraphiQL = exports.shouldRenderGraphiQL = void 0;
const tslib_1 = require("tslib");
const utils_1 = require("@graphql-tools/utils");
const graphiql_html_js_1 = tslib_1.__importDefault(require("../graphiql-html.js"));
function shouldRenderGraphiQL({ headers, method }) {
    return method === 'GET' && !!headers?.get('accept')?.includes('text/html');
}
exports.shouldRenderGraphiQL = shouldRenderGraphiQL;
const renderGraphiQL = (opts) => graphiql_html_js_1.default
    .replace('__TITLE__', opts?.title || 'Yoga GraphiQL')
    .replace('__OPTS__', JSON.stringify(opts ?? {}));
exports.renderGraphiQL = renderGraphiQL;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function useGraphiQL(config) {
    const logger = config.logger ?? console;
    let graphiqlOptionsFactory;
    if (typeof config?.options === 'function') {
        graphiqlOptionsFactory = config?.options;
    }
    else if (typeof config?.options === 'object') {
        graphiqlOptionsFactory = () => config?.options;
    }
    else if (config?.options === false) {
        graphiqlOptionsFactory = () => false;
    }
    else {
        graphiqlOptionsFactory = () => ({});
    }
    const renderer = config?.render ?? exports.renderGraphiQL;
    let urlPattern;
    const getUrlPattern = ({ URLPattern }) => {
        urlPattern ||= new URLPattern({
            pathname: config.graphqlEndpoint,
        });
        return urlPattern;
    };
    return {
        onRequest({ request, serverContext, fetchAPI, endResponse, url }) {
            if (shouldRenderGraphiQL(request) &&
                (request.url.endsWith(config.graphqlEndpoint) ||
                    url.pathname === config.graphqlEndpoint ||
                    getUrlPattern(fetchAPI).test(url))) {
                logger.debug(`Rendering GraphiQL`);
                function processGraphiQLOptions(graphiqlOptions) {
                    if (graphiqlOptions) {
                        function processGraphiQLBody(graphiQLBody) {
                            const response = new fetchAPI.Response(graphiQLBody, {
                                headers: {
                                    'Content-Type': 'text/html',
                                },
                                status: 200,
                            });
                            endResponse(response);
                        }
                        const graphiQLBody$ = renderer({
                            ...(graphiqlOptions === true ? {} : graphiqlOptions),
                        });
                        if ((0, utils_1.isPromise)(graphiQLBody$)) {
                            return graphiQLBody$.then(processGraphiQLBody);
                        }
                        return processGraphiQLBody(graphiQLBody$);
                    }
                }
                const graphiqlOptions$ = graphiqlOptionsFactory(request, serverContext);
                if ((0, utils_1.isPromise)(graphiqlOptions$)) {
                    return graphiqlOptions$.then(processGraphiQLOptions);
                }
                return processGraphiQLOptions(graphiqlOptions$);
            }
        },
    };
}
exports.useGraphiQL = useGraphiQL;
