import { isSchema } from 'graphql';
import { isPromise } from '@whatwg-node/server';
export const useSchema = (schemaDef) => {
    if (schemaDef == null) {
        return {};
    }
    if (isSchema(schemaDef)) {
        return {
            onPluginInit({ setSchema }) {
                setSchema(schemaDef);
            },
        };
    }
    if ('then' in schemaDef) {
        let schema;
        return {
            onRequestParse() {
                return {
                    onRequestParseDone() {
                        return schemaDef.then(s => {
                            schema = s;
                        });
                    },
                };
            },
            onEnveloped({ setSchema }) {
                if (!schema) {
                    throw new Error(`You provide a promise of a schema but it hasn't been resolved yet. Make sure you use this plugin with GraphQL Yoga.`);
                }
                setSchema(schema);
            },
        };
    }
    const schemaByRequest = new WeakMap();
    return {
        onRequestParse({ request, serverContext }) {
            return {
                onRequestParseDone() {
                    const schema$ = schemaDef({
                        ...serverContext,
                        request,
                    });
                    if (isPromise(schema$)) {
                        return schema$.then(schema => {
                            schemaByRequest.set(request, schema);
                        });
                    }
                    schemaByRequest.set(request, schema$);
                },
            };
        },
        onEnveloped({ setSchema, context }) {
            if (context?.request == null) {
                throw new Error('Request object is not available in the context. Make sure you use this plugin with GraphQL Yoga.');
            }
            const schema = schemaByRequest.get(context.request);
            if (schema == null) {
                throw new Error(`No schema found for this request. Make sure you use this plugin with GraphQL Yoga.`);
            }
            setSchema(schema);
        },
    };
};
