import { getOperationAST } from 'graphql';
import { isPromise } from '@graphql-tools/utils';
import { iterateAsyncVoid } from '@whatwg-node/server';
export function processResult({ request, result, fetchAPI, onResultProcessHooks, }) {
    let resultProcessor;
    const acceptableMediaTypes = [];
    let acceptedMediaType = '*/*';
    function executeResultProcessor() {
        // If no result processor found for this result, return an error
        if (!resultProcessor) {
            return new fetchAPI.Response(null, {
                status: 406,
                statusText: 'Not Acceptable',
                headers: {
                    accept: acceptableMediaTypes.join('; charset=utf-8, '),
                },
            });
        }
        return resultProcessor(result, fetchAPI, acceptedMediaType);
    }
    const iterationRes$ = iterateAsyncVoid(onResultProcessHooks, onResultProcessHook => onResultProcessHook({
        request,
        acceptableMediaTypes,
        result,
        setResult(newResult) {
            result = newResult;
        },
        resultProcessor,
        setResultProcessor(newResultProcessor, newAcceptedMimeType) {
            resultProcessor = newResultProcessor;
            acceptedMediaType = newAcceptedMimeType;
        },
    }));
    if (isPromise(iterationRes$)) {
        return iterationRes$.then(executeResultProcessor);
    }
    return executeResultProcessor();
}
function processExecutionArgs({ executionArgs, enveloped, }) {
    // Get the actual operation
    const operation = getOperationAST(executionArgs.document, executionArgs.operationName);
    // Choose the right executor
    const executeFn = operation?.operation === 'subscription' ? enveloped.subscribe : enveloped.execute;
    // Get the result to be processed
    return executeFn(executionArgs);
}
export function processRequest({ params, enveloped, }) {
    // Parse GraphQLParams
    const document = enveloped.parse(params.query);
    // Validate parsed Document Node
    const errors = enveloped.validate(enveloped.schema, document);
    if (errors.length > 0) {
        return { errors };
    }
    function processContextValue(contextValue) {
        const executionArgs = {
            schema: enveloped.schema,
            document,
            contextValue,
            variableValues: params.variables,
            operationName: params.operationName,
        };
        return processExecutionArgs({ executionArgs, enveloped });
    }
    // Build the context for the execution
    const contextValue$ = enveloped.contextFactory();
    if (isPromise(contextValue$)) {
        return contextValue$.then(processContextValue);
    }
    return processContextValue(contextValue$);
}
